module Urbivore
  class Message
    attr_reader :timestamp, :message, :level, :progname, :data

    # generally speaking we don't want users initializing
    # these directly---either Logger or MessagePacket should 
    # be doing that in most cases
    def initialize(timestamp, message, level, progname = nil, data = nil)
      # that said, let's raise an error if at least they're
      # calling this wrong... 
      unless timestamp.is_a?(Time)
        raise Urbivore::Exceptions::TypeError.new("Message expected timestamp to be a time")
      end

      @timestamp  = timestamp
      @message    = message
      @level      = level.to_sym
      @progname   = progname
      @data       = data
    end

    def to_s
      str = "#{timestamp} [#{level}] #{message}"
      progname ? str += " (#{progname})" : str
    end

    def <=>(comp)
      if self.timestamp < comp.timestamp
        -1
      elsif self.timestamp > comp.timestamp
        1
      else
        0
      end
    end
  end
end
